--------------------------------------------------------------------------------
-- Title   : clock sources
-- Project : TimEX3
--------------------------------------------------------------------------------
-- Description : sub frequecy clocks generated from the main clock
--------------------------------------------------------------------------------
-- File     : clk_sources.vhd
-- Revision : x.x.x
-- Created  : October 30, 2012
-- Updated  : April 30, 2015
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2012 - 2013 Synchrotron Soleil
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

Library UNISIM;
use UNISIM.vcomponents.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity clk_sources is
  port (
    p_clk60MHz  : in  std_logic; -- clock 60 MHz
    p_clk240MHz : out std_logic; -- clock 120 MHz
    p_reset     : in  std_logic; -- reset input
    p_clk1kHz   : out std_logic; -- clock 0.1 Hz
    p_clk500mHz : out std_logic  -- clock 0.5 Hz
  );
end entity clk_sources;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_clk_sources of clk_sources is
  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------
  constant c_lfsr   : std_logic_vector (15 downto 0) := "0101011011101011";
  constant c_cntmax : unsigned (8 downto 0) := "111110011"; -- /500
  constant c_cntH   : unsigned (7 downto 0) := "11001000";

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  signal s_lfsr      : std_logic_vector (15 downto 0);
  signal s_lfsr_lsb  : std_logic;
  signal s_cnt       : unsigned (8 downto 0); -- counter
  signal s_clk1kHz   : std_logic;
  signal s_clk500mHz : std_logic;
  signal GND_BIT     : std_logic;


  ------------------------------------------------------------------------------
  -- Attributes
  ------------------------------------------------------------------------------
  attribute PERIOD : string;
  attribute PERIOD of p_clk240MHz : signal is "4 ns";

  attribute BUFFER_TYPE : string;
  attribute BUFFER_TYPE of p_clk240MHz : signal is "BUFG";

 ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  ------------------------------------------------------------------------------
  -- 1 kHz clock
  process (p_clk60MHz, p_reset)
  begin
    if (p_reset = '1') then
      s_lfsr <= (OTHERS => '0');
      s_clk1kHz <= '0';
    elsif (rising_edge(p_clk60MHz)) then
      if(s_lfsr = c_lfsr) then
        s_lfsr <= (OTHERS => '0');
        s_clk1kHz <= '1';
      else
        s_lfsr <= s_lfsr(14 downto 0) & s_lfsr_lsb;
        s_clk1kHz <= '0';
      end if;
    end if;
  end process;

  ------------------------------------------------------------------------------
  -- 0.5 Hz clock
  process (p_reset, s_clk1kHz)
  begin
    if (p_reset = '1') then
      s_cnt <= (OTHERS => '0');
    elsif (rising_edge(s_clk1kHz)) then
      if (s_cnt < c_cntmax) then
        s_cnt <= s_cnt + 1;
      else
        s_cnt <= (OTHERS => '0');
      end if;
    end if;
  end process;

  process (p_reset, s_clk1kHz)
  begin
    if (p_reset = '1') then
      s_clk500mHz <= '1';
    elsif (rising_edge(s_clk1kHz)) then
      if (s_cnt < c_cntH) then
        s_clk500mHz <= '1';
      else
        s_clk500mHz <= '0';
      end if;
    end if;
  end process;

  ------------------------------------------------------------------------------
  -- 120 MHz clock
  -- DCM_SP: Digital Clock Manager
  --         Spartan-6
  -- Xilinx HDL Language Template, version 14.4

  GND_BIT <= '0';

  DCM_SP_inst : DCM_SP
  generic map (
     CLKDV_DIVIDE          => 2.0,                  -- CLKDV divide value
                                                    -- (1.5,2,2.5,3,3.5,4,4.5,5,5.5,6,6.5,7,7.5,8,9,10,11,12,13,14,15,16).
     CLKFX_DIVIDE          => 1,                    -- Divide value on CLKFX outputs - D - (1-32)
     CLKFX_MULTIPLY        => 4,                    -- Multiply value on CLKFX outputs - M - (2-32)
     CLKIN_DIVIDE_BY_2     => FALSE,                -- CLKIN divide by two (TRUE/FALSE)
     CLKIN_PERIOD          => 16.7,                 -- Input clock period specified in nS
     CLKOUT_PHASE_SHIFT    => "NONE",               -- Output phase shift (NONE, FIXED, VARIABLE)
     CLK_FEEDBACK          => "NONE",               -- Feedback source (NONE, 1X, 2X)
     DESKEW_ADJUST         => "SYSTEM_SYNCHRONOUS", -- SYSTEM_SYNCHRNOUS or SOURCE_SYNCHRONOUS
     DFS_FREQUENCY_MODE    => "LOW",                -- Unsupported - Do not change value
     DLL_FREQUENCY_MODE    => "LOW",                -- Unsupported - Do not change value
     DSS_MODE              => "NONE",               -- Unsupported - Do not change value
     DUTY_CYCLE_CORRECTION => TRUE,                 -- Unsupported - Do not change value
     FACTORY_JF            => X"c080",              -- Unsupported - Do not change value
     PHASE_SHIFT           => 0,                    -- Amount of fixed phase shift (-255 to 255)
     STARTUP_WAIT          => FALSE                 -- Delay config DONE until DCM_SP LOCKED (TRUE/FALSE)
  )
  port map (
     CLK0     => open,        -- 1-bit output: 0 degree clock output
     CLK180   => open,        -- 1-bit output: 180 degree clock output
     CLK270   => open,        -- 1-bit output: 270 degree clock output
     CLK2X    => open,        -- 1-bit output: 2X clock frequency clock output
     CLK2X180 => open,        -- 1-bit output: 2X clock frequency, 180 degree clock output
     CLK90    => open,        -- 1-bit output: 90 degree clock output
     CLKDV    => open,        -- 1-bit output: Divided clock output
     CLKFX    => p_clk240MHz, -- 1-bit output: Digital Frequency Synthesizer output (DFS)
     CLKFX180 => open,        -- 1-bit output: 180 degree CLKFX output
     LOCKED   => open,        -- 1-bit output: DCM_SP Lock Output
     PSDONE   => open,        -- 1-bit output: Phase shift done output
     STATUS   => open,        -- 8-bit output: DCM_SP status output
     CLKFB    => GND_BIT,     -- 1-bit input: Clock feedback input
     CLKIN    => p_clk60MHz,  -- 1-bit input: Clock input
     DSSEN    => GND_BIT,     -- 1-bit input: Unsupported, specify to GND.
     PSCLK    => GND_BIT,     -- 1-bit input: Phase shift clock input
     PSEN     => GND_BIT,     -- 1-bit input: Phase shift enable
     PSINCDEC => GND_BIT,     -- 1-bit input: Phase shift increment/decrement input
     RST      => p_reset      -- 1-bit input: Active high reset input
  );

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
  s_lfsr_lsb  <= s_lfsr(15) xnor s_lfsr(14) xnor s_lfsr(12) xnor s_lfsr(3);
  p_clk1kHz   <= s_clk1kHz;
  p_clk500mHz <= s_clk500mHz;

end architecture rtl_clk_sources;
