--------------------------------------------------------------------------------
-- Title   : Top - clock padding function
-- Project : TimEX3
--------------------------------------------------------------------------------
-- Description : generate a 1 kHz clock on the output whenever 1 kHz input
--               clock is missing.
--               It is used by Cristal laser synchro system to avoid gaps in the
--               1 kHz clock when changing delays
--               The laser frequency is :
--                   Fl = Frf / 2 / 0x2AF60 = 1000.7397555 Hz (999.261 s)
--               The input is on pin_inTTL(0) ; the output is on pin_outTTL(2)
--
--               generate a 5 kHz clock on the output whenever 5 kHz input
--               clock is missing.
--               It is used by Cristal laser synchro system to avoid gaps in the
--               5 kHz clock when changing delays
--               The laser frequency is :
--                   Fl = Frf / 2 / 0x8950 = 5009.701 Hz (199.613 s)
--               The input is on pin_inTTL(1) ; the output is on pin_outTTL(3)
--------------------------------------------------------------------------------
-- File     : clkpadding_top.vhdl
-- Revision : x.x.x
-- Created  : July 19, 2014
-- Updated  : October 04, 2018
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2012 - 2018 Synchrotron Soleil
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the Lesser GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity clkPADDING_top is
  port (
    p_clkPADDING_clk500mHz : in  std_logic;
    p_clkPADDING_clk60MHz  : in  std_logic;
    p_clkPADDING_1kHz_mon  : in  std_logic; -- 1 kHz monitored clock
    p_clkPADDING_5kHz_mon  : in  std_logic; -- 5 kHz monitored clock
    p_clkPADDING_reset     : in  std_logic; -- global reset
    p_clkPADDING_outTTL    : out std_logic_vector (4 downto 0); -- TTL outputs  -- output clock
    p_clkPADDING_outPECL   : out std_logic_vector (4 downto 0); -- LVPECL outputs
    p_clkPADDING_LED       : out std_logic_vector (1 downto 0)
  );
end entity clkPADDING_top;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_clkPADDING_top of clkPADDING_top is

  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  signal s_clkPADDING_LED_1kHz : std_logic := '0';
  signal s_clkPADDING_LED_5kHz : std_logic := '0';

 ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  -- 1kHz clock padding
  padding_1kHz : entity work.clkpadding_padding (rtl_clkpadding_padding)
  generic map (
    g_gap_detect     => X"EA36", -- 59958 ; (59958 + 1) / 60e6 = 999.317 s
                                 -- 1 kHz period = 999.261 s
    g_padding_period => X"EA34", -- 59956 ; 59956 / 60e6 = 999.267 s
    g_pulse_high     => X"2EE0") -- 12000 => 200 s
  port map (
    p_clkPADDING_clk500mHz => p_clkPADDING_clk500mHz,
    p_clkPADDING_clk60MHz  => p_clkPADDING_clk60MHz,
    p_clkPADDING_clk_mon   => p_clkPADDING_1kHz_mon,  -- monitored clock
    p_clkPADDING_reset     => p_clkPADDING_reset,     -- global reset
    p_clkPADDING_outTTL    => p_clkPADDING_outTTL(2), -- TTL outputs ; output clock
    p_clkPADDING_LED       => s_clkPADDING_LED_1kHz
  );

  -- 5kHz clock padding
  padding_5kHz : entity work.clkpadding_padding (rtl_clkpadding_padding)
  generic map (
    g_gap_detect     => X"2ED2",  -- 11986 ; (11986 + 1) / 60e6 = 199.783 s
                                  -- 5 kHz period = 199.613 s
    g_padding_period => X"2EC9",  -- 11977 ; 11977 / 60e6 = 199.617 s
    g_pulse_high     => X"1518")  -- 5400 => 90 s
  port map (
    p_clkPADDING_clk500mHz => p_clkPADDING_clk500mHz,
    p_clkPADDING_clk60MHz  => p_clkPADDING_clk60MHz,
    p_clkPADDING_clk_mon   => p_clkPADDING_5kHz_mon,  -- monitored clock
    p_clkPADDING_reset     => p_clkPADDING_reset,     -- global reset
    p_clkPADDING_outTTL    => p_clkPADDING_outTTL(3), -- TTL outputs ; output clock
    p_clkPADDING_LED       => s_clkPADDING_LED_5kHz
  );

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------

  p_clkPADDING_outTTL(0) <= '0'; -- not used ; is configured as input
  p_clkPADDING_outTTL(1) <= '0'; -- not used ; is configured as input
  p_clkPADDING_outTTL(4) <= '0'; -- not used

  p_clkPADDING_outPECL(4 downto 0) <= "00000";

  p_clkPADDING_LED(0) <= s_clkPADDING_LED_1kHz and s_clkPADDING_LED_5kHz; -- red LED
  p_clkPADDING_LED(1) <= '1'; -- green LED ON

end architecture rtl_clkPADDING_top;
