--------------------------------------------------------------------------------
-- Title   : Monitoring of CLK_SR input
-- Project : TimEX3
--------------------------------------------------------------------------------
-- Description : The CLK_SR signal is monotored at a frequency of 1kHz.
--               A 32 bits conter is used to save the time between two synchroization
--               pulses.
--------------------------------------------------------------------------------
-- File     : freqDIV_monitoring.vhdl
-- Revision : x.x.x
-- Created  : February 03, 2014
-- Updated  : February 03, 2014
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2012 - 2014 Synchrotron Soleil
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity freqDIV_monitoring is
  port (
    p_inTTL   : in  std_logic;                     -- TTL input
    p_clk1kHz : in  std_logic;
    p_reset   : in  std_logic;
    p_monitor : out std_logic_vector (31 downto 0) -- register with the time
                                                   -- value between two pulses
  );
end freqDIV_monitoring;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_freqDIV_monitoring of freqDIV_monitoring is

  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------
  constant c_cntmax : unsigned (31 downto 0) := X"FFFFFFFF";

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  signal s_cnt   : unsigned (31 downto 0); -- counter
  signal s_start : std_logic;
  signal s_stop  : std_logic;
  signal s_hold1 : std_logic;
  signal s_hold2 : std_logic;

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  process (p_reset, p_inTTL, s_stop)
  begin
    if ((s_stop = '1') or (p_reset = '1')) then
      s_start <= '0';
    elsif (rising_edge(p_inTTL)) then
      s_start <= '1';
    end if;
  end process;

  process (p_reset, s_start, p_clk1kHz)
  begin
    if ((s_start = '0') or (p_reset = '1')) then
      s_hold1 <= '0';
      s_hold2 <= '0';
      s_stop  <= '0';
    elsif (falling_edge(p_clk1kHz)) then
      s_hold1 <= '1';
      s_hold2 <= s_hold1;
      s_stop  <= s_hold2;
    end if;
  end process;

  process (p_reset, s_stop, p_clk1kHz)
  begin
    if ((s_stop = '1') or (p_reset = '1')) then
      s_cnt <= (OTHERS => '0');
    elsif (falling_edge(p_clk1kHz)) then
      if (s_cnt < c_cntmax) then
        s_cnt <= s_cnt + 1;
      else
        s_cnt <= s_cnt; -- keep the max value in case of overflow
      end if;
    end if;
  end process;

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
  p_monitor <= std_logic_vector(s_cnt);

end rtl_freqDIV_monitoring;
